<?php
/*
  * Plugin Name:       Simple Assistant AI
  * Plugin URI:        https://freefuture.eu/download/sample-plugin-ai.zip
  * Description:       Handle the basics with this assistant AI.
  * Version:           1.9
  * Requires at least: 5.2
  * Requires PHP:      7.2
  * Author:            Massimo Murri for Free Future
  * Author URI:        https://freefuture.eu/
  * License: GPLv2 or later
  * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
  * Sample Assistant AI - AI Integration
  * Copyright (c) 2024 Free Future Lda
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Define the encrypted API key
define('SIMPLE_ASSISTANT_AI_API_KEY_ENCRYPTED', 'your-encrypted-key-here'); // Replace with your encrypted key

// XOR encryption/decryption function
function xor_encrypt_decrypt($input, $key) {
    $output = '';
    for ($i = 0; $i < strlen($input); $i++) {
        $output .= $input[$i] ^ $key[$i % strlen($key)];
    }
    return $output;
}

// Decrypt the API key
function get_decrypted_api_key() {
    $encrypted_key = base64_decode(SIMPLE_ASSISTANT_AI_API_KEY_ENCRYPTED);
    $key = 'some_secret_key'; // Must match the key used during encryption
    return xor_encrypt_decrypt($encrypted_key, $key);
}

// Add a settings page in the admin menu
add_action('admin_menu', 'simple_assistant_ai_menu');
function simple_assistant_ai_menu() {
    add_menu_page('Assistant AI', 'Assistant AI', 'manage_options', 'assistant-ai', 'simple_assistant_ai_settings_page');
}

function simple_assistant_ai_settings_page() {
    ?>
    <div class="wrap">
        <h1>Assistant AI Settings</h1>
        <p>This is a trial version that will let you test the plugin.<br> Use this shortcode to add the AI in any place you prefer show it, works with any website builder [simple_assistant_ai] <br>You can ask anything, allowed 10 answers a day.</br> You can add your own rules in the assistant rules field to personalize the AI. <br>You can check and use some examples by copying them from <a style="font-weight:bold" href="https://freefuture.eu/documentazione/">DOCS and SAMPLE</a>, and pasting them into the assistant rules field here.<br>For full control, update the plugin here: <a style="font-weight:bold" href="https://freefuture.eu/#section-5">Plugin AI by FREE FUTURE-Prices</a></p>
        <form method="post" action="options.php">
            <?php
            settings_fields('simple_assistant_ai_options_group');
            do_settings_sections('assistant-ai');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Register and define the settings
add_action('admin_init', 'simple_assistant_ai_settings');
function simple_assistant_ai_settings() {
    register_setting('simple_assistant_ai_options_group', 'simple_assistant_ai_rules');
    register_setting('simple_assistant_ai_options_group', 'simple_assistant_ai_name');

    add_settings_section('simple_assistant_ai_section', 'Settings', null, 'assistant-ai');

    add_settings_field('simple_assistant_ai_rules', 'Assistant Rules', 'simple_assistant_ai_rules_callback', 'assistant-ai', 'simple_assistant_ai_section');
    add_settings_field('simple_assistant_ai_name', 'Assistant Name', 'simple_assistant_ai_name_callback', 'assistant-ai', 'simple_assistant_ai_section');
}

function simple_assistant_ai_rules_callback() {
    $simple_assistant_ai_rules = get_option('simple_assistant_ai_rules');
    echo '<textarea name="simple_assistant_ai_rules" rows="5" cols="50">' . esc_textarea($simple_assistant_ai_rules) . '</textarea>';
}

function simple_assistant_ai_name_callback() {
    $simple_assistant_ai_name = get_option('simple_assistant_ai_name');
    echo '<input type="text" name="simple_assistant_ai_name" placeholder="Name your assistant (optional)" value="' . esc_attr($simple_assistant_ai_name) . '" />';
}

// Function to generate responses using OpenAI with decrypted API key
function generate_simple_assistant_ai_response($prompt, $conversation_history) {
    // Use the decrypted API key
    $api_key = get_decrypted_api_key();

    // Retrieve the user's rules
    $user_rules = get_option('simple_assistant_ai_rules');

    $url = 'https://api.openai.com/v1/chat/completions';
    $messages = [
        ['role' => 'system', 'content' => $user_rules], // Apply the rules internally
        ['role' => 'user', 'content' => $prompt]
    ];

    // Append conversation history if any
    if (!empty($conversation_history)) {
        foreach ($conversation_history as $message) {
            $messages[] = ['role' => 'user', 'content' => $message['user']];
            $messages[] = ['role' => 'assistant', 'content' => $message['assistant']];
        }
    }

    $data = [
        'model' => 'gpt-4', // Use the GPT-4 model
        'messages' => $messages,
        'max_tokens' => 500,
    ];

    $response = wp_remote_post($url, [
        'headers' => [
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer ' . $api_key
        ],
        'body' => wp_json_encode($data),
        'timeout' => 30, // Increase the timeout to 30 seconds
    ]);

    if (is_wp_error($response)) {
        error_log('Error in request: ' . $response->get_error_message());
        return 'Error in request: ' . $response->get_error_message();
    }

    $body = wp_remote_retrieve_body($response);
    $result = json_decode($body, true);

    if (isset($result['choices']) && is_array($result['choices']) && isset($result['choices'][0]['message']['content'])) {
        $assistant_response = trim($result['choices'][0]['message']['content']);

        // Trial message to inform the user
        $trial_message = "This is a trial version. <br>For full control, update at: <a style=\"font-weight:bold\" href=\"https://www.freefuture.eu\">FREE FUTURE LDA</a>";

        // Append only the trial message to the response
        $final_response = $assistant_response . "\n\n" . $trial_message;

        // Return the assistant's response
        return $final_response;
    } else {
        error_log('No phrase generated. Response: ' . print_r($result, true));
        return 'No phrase generated.';
    }
}

// Function to track the number of responses per day per user
function track_user_response_limit() {
    $user_ip = $_SERVER['REMOTE_ADDR']; // Track by user IP, or use a logged-in user ID if available
    $option_name = 'simple_assistant_ai_' . md5($user_ip);

    // Get current tracking data
    $user_data = get_option($option_name);
    $current_time = time();

    // If no data exists or it's been more than 24 hours, reset the data
    if (!$user_data || ($current_time - $user_data['start_time']) > 86400) {
        $user_data = [
            'count' => 0,
            'start_time' => $current_time,
        ];
    }

    // Increment the counter
    $user_data['count']++;

    // Update the option with the new data
    update_option($option_name, $user_data);

    return $user_data;
}

// Check if user has reached the daily limit
function has_user_reached_daily_limit() {
    $user_data = track_user_response_limit();
    return $user_data['count'] > 10;
}

// Enqueue jQuery and inline script
add_action('wp_enqueue_scripts', 'simple_assistant_ai_enqueue_scripts');
function simple_assistant_ai_enqueue_scripts() {
    wp_enqueue_script('jquery');
    add_action('wp_footer', 'simple_assistant_ai_inline_script');
}

function simple_assistant_ai_inline_script() {
    $assistant_name = get_option('simple_assistant_ai_name', 'Assistant'); // Default to 'Assistant' if no name is set
    $nonce = wp_create_nonce('simple_assistant_ai_nonce');
    ?>
    <style>
        .status-bar {
            display: none;
            position: absolute;
            height: 40px; /* Same height as the submit button */
            background-color: blue;
            z-index: 9999;
            transition: width 20s linear; /* Fill over 20 seconds */
        }

        .button-wrapper {
            position: relative;
            display: block;
            text-align: center;
        }

        .button-wrapper button {
            width: 25%;
            padding: 10px;
        }

        /* Add a 1px solid black border around the input field */
        #simple-assistant-ai-prompt {
            border: 1px solid black;
            width: 100%;
            padding: 10px;
            box-sizing: border-box;
            margin-bottom: 10px;
        }

        /* Add some margin and padding to the overall form */
        #simple-assistant-ai-form {
            margin: 20px;
            padding: 20px;
            border: 1px solid #ccc;
            max-width: 600px;
            margin: 0 auto; /* Center the form */
        }

        #simple-assistant-ai-response {
            max-height: 500px;
            overflow-y: auto;
            margin-bottom: 20px;
            padding: 10px;
            border: 1px solid #ccc;
        }
    </style>
    <script type="text/javascript">
    jQuery(document).ready(function($) {
        var conversationHistory = [];

        function submitPrompt() {
            var prompt = $('#simple-assistant-ai-prompt').val();
            var responseDiv = $('#simple-assistant-ai-response');
            var submitButton = $('#simple-assistant-ai-submit');
            var statusBar = $('<div class="status-bar"></div>');

            $('.button-wrapper').append(statusBar);
            statusBar.css({
                top: submitButton.position().top,
                left: submitButton.position().left,
                width: 0, // Start width at 0
                height: submitButton.outerHeight()
            }).show();

            // Animate the width over 20 seconds
            setTimeout(function() {
                statusBar.css('width', submitButton.outerWidth());
            }, 0);

            submitButton.prop('disabled', true);

            var data = {
                'action': 'simple_assistant_ai_generate_response',
                'prompt': prompt,
                'conversation_history': JSON.stringify(conversationHistory),
                'nonce': '<?php echo esc_js($nonce); ?>'
            };

            $.post('<?php echo admin_url('admin-ajax.php'); ?>', data, function(response) {
                // Stop the animation and set the width to 100%
                statusBar.css('width', '100%');

                // Append user prompt and assistant response to the conversation history
                conversationHistory.push({ 'user': prompt, 'assistant': response });

                // Display the conversation
                var conversationHtml = '';
                conversationHistory.forEach(function(entry) {
                    conversationHtml += '<p><strong>You:</strong> ' + entry.user + '</p>';
                    conversationHtml += '<p><strong><?php echo esc_js($assistant_name); ?>:</strong> ' + entry.assistant + '</p>';
                });

                responseDiv.html(conversationHtml);
                $('#simple-assistant-ai-prompt').val(''); // Clear the input field
                responseDiv.scrollTop(responseDiv[0].scrollHeight); // Scroll to the bottom
                statusBar.fadeOut(400, function() { $(this).remove(); });
                submitButton.prop('disabled', false);
            });
        }

        $('#simple-assistant-ai-submit').on('click', function() {
            submitPrompt();
        });

        $('#simple-assistant-ai-prompt').on('keypress', function(e) {
            if (e.which == 13) { // Enter key pressed
                e.preventDefault();
                submitPrompt();
            }
        });
    });
    </script>
    <?php
}

// Shortcode to display the assistant
add_shortcode('simple_assistant_ai', 'simple_assistant_ai_shortcode');
function simple_assistant_ai_shortcode() {
    ob_start();
    ?>
    <div id="simple-assistant-ai-form">
        <div id="simple-assistant-ai-response" style="max-height: 500px; overflow-y: auto; margin-bottom: 20px; padding: 10px; border: 1px solid #ccc;"></div>
        <input type="text" id="simple-assistant-ai-prompt" placeholder="Ask me anything..." style="width: 100%; padding: 10px; box-sizing: border-box; margin-bottom: 10px;">
        <div class="button-wrapper text-center">
            <button id="simple-assistant-ai-submit" style="padding: 10px;">Submit</button>
        </div>
    </div>
    <?php
    return ob_get_clean();
}

// Handle the AJAX request for generating response
add_action('wp_ajax_simple_assistant_ai_generate_response', 'simple_assistant_ai_generate_response');
add_action('wp_ajax_nopriv_simple_assistant_ai_generate_response', 'simple_assistant_ai_generate_response');
function simple_assistant_ai_generate_response() {
    if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'simple_assistant_ai_nonce')) {
        wp_die('Nonce verification failed.');
    }

    if (has_user_reached_daily_limit()) {
        echo wp_kses_post("This is a trial version that will let you test the plugin, will allow you 10 answer a day, you can add on the admin panel your own rules.<br>For full control, update the plugin here: <a style=\"font-weight:bold\" href=\"https://www.freefuture.eu\">FREE FUTURE LDA</a>");
    } else {
        if (isset($_POST['prompt']) && isset($_POST['conversation_history'])) {
            $prompt = sanitize_text_field($_POST['prompt']);
            $conversation_history = json_decode(stripslashes($_POST['conversation_history']), true);
            $response = generate_simple_assistant_ai_response($prompt, $conversation_history);
            echo wp_kses_post($response);
        } else {
            echo 'Invalid request.';
        }
    }
    wp_die();
}
